Object subclass: #NmWorldMap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NM-WorldMap'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NmWorldMap class
	instanceVariableNames: 'map mapPath localPoint '!

!NmWorldMap class methodsFor: 'accessing'!
map
	^ map! !

!NmWorldMap class methodsFor: 'accessing'!
map: aDictionary
	map _ aDictionary! !


!NmWorldMap class methodsFor: 'class initialization' stamp: 'MU 2/25/2003 18:33'!
initialize
	"NmWorldMap initialize"
	map _ Dictionary new.
	localPoint _ 0@0
	
	
	! !


!NmWorldMap class methodsFor: 'storing/loading' stamp: 'my 4/9/2003 22:32'!
listLoadableMapNames
	"NmWorldMap listLoadableMapNames"
	NmSettings useExternalAdminServer
		ifTrue: [| call url resp port | 
			url _ NmSettings adminServerUrl asUrl.
			port _ url port.
			call _ (SoapCallEntry
						tcpHost: url authority
						port: (port isNil
								ifTrue: [NmSettings defaultAdminServerPort]
								ifFalse: [port])) newCall.
			call targetObjectURI: url fullPath.
			call methodName: 'listAllMapNames'.
			resp _ call invoke.
			^ resp isFault
				ifTrue: [resp raiseException.
					nil]
				ifFalse: [resp returnValue]]
		ifFalse: [^ NmWorldInfoReadWriter listMapCsvFileNamesFromDirectory]! !

!NmWorldMap class methodsFor: 'storing/loading' stamp: 'mu 3/19/2003 01:49'!
load
	"NmWorldMap load"
	| mapString |
	NmSettings useExternalAdminServer
		ifTrue: [| call url resp port | 
			url _ NmSettings adminServerUrl asUrl.
			port _ url port.
			call _ (SoapCallEntry
						tcpHost: url authority
						port: (port isNil
								ifTrue: [NmSettings defaultAdminServerPort]
								ifFalse: [port])) newCall.
			call targetObjectURI: url fullPath.
			call methodName: 'getMapStringNamed:'.
			call
				addParameter: (SoapVariable name: #mapName value: NmSettings mapName).
			resp _ call invoke.
			mapString _ resp isFault
						ifTrue: [resp raiseException.
							nil]
						ifFalse: [resp returnValue].
			self fromCSVString: mapString]
		ifFalse: [mapString _ NmWorldInfoReadWriter getCsvStringFromFile: NmSettings mapName.
			self fromCSVString: mapString].
	Transcript cr; show: '## NmWorldMap was loaded: ' , self map printString.
	self resetLocal! !

!NmWorldMap class methodsFor: 'storing/loading' stamp: 'mu 3/24/2003 01:21'!
loadAndUpdate	
	self load.
	NmResolver loadAndUpdate.! !

!NmWorldMap class methodsFor: 'storing/loading' stamp: 'mu 3/17/2003 01:00'!
store
	"NmWorldMap store"
	NmSettings useExternalAdminServer
		ifTrue: [| call url resp port | 
			url _ NmSettings adminServerUrl asUrl.
			port _ url port.
			call _ (SoapCallEntry
						tcpHost: url authority
						port: (port isNil
								ifTrue: [NmSettings defaultAdminServerPort]
								ifFalse: [port])) newCall.
			call targetObjectURI: url fullPath.
			call methodName: 'putMapString:named:'.
			call
				addParameter: (SoapVariable name: #mapCSVString value: self csvString).
			call
				addParameter: (SoapVariable name: #mapName value: NmSettings mapName).
			resp _ call invoke.
			resp isFault
				ifTrue: [resp raiseException]
				ifFalse: [resp returnValue]]
		ifFalse: [NmWorldInfoReadWriter  putCsvString: self csvString toFile: NmSettings mapName].
	self resetLocal! !


!NmWorldMap class methodsFor: 'accessing point' stamp: 'mu 1/5/2003 23:19'!
bottomOf: address 
	"NmWorldMap bottomOf: 'nm://smalltalk:37458/' asUrl"
	| point destinationPoint |
	point _ self pointOf: address.
	point isNil
		ifTrue: [^ nil].
	destinationPoint _ Point x: point x y: point y - 1.
	^ map
		at: destinationPoint
		ifAbsent: [^ nil]! !

!NmWorldMap class methodsFor: 'accessing point' stamp: 'mu2 11/25/2002 17:47'!
deltaPointFrom: fromUrl to: toUrl 
	"NmWorldMap deltaPointFrom: 'nm://lisp:37458' asUrl to: 
	'nm://lisp:37459' asUrl  800@0"
	| fromPoint toPoint |
	fromPoint _ self pointOf: fromUrl.
	toPoint _ self pointOf: toUrl.
	^ toPoint - fromPoint * (ActiveWorld extent) * (1 @ -1)! !

!NmWorldMap class methodsFor: 'accessing point' stamp: 'mu 1/5/2003 23:19'!
leftOf: address 
	"NmWorldMap leftOf: 'nm://smalltalk:37458/' asUrl"
	| point destinationPoint |
	point _ self pointOf: address.
	point isNil
		ifTrue: [^ nil].
	destinationPoint _ Point x: point x - 1 y: point y.
	^ map
		at: destinationPoint
		ifAbsent: [^ nil]! !

!NmWorldMap class methodsFor: 'accessing point' stamp: 'mu 3/13/2003 18:29'!
localPoint
	^localPoint! !

!NmWorldMap class methodsFor: 'accessing point' stamp: 'mu 3/19/2003 01:11'!
localPoint: aPoint
	localPoint _ aPoint! !

!NmWorldMap class methodsFor: 'accessing point' stamp: 'mu 3/23/2003 12:35'!
pointOf: address 
	"NmWorldMap pointOf: 'nm://Smalltalk:37458/' asUrl"
	"NmWorldMap pointOf: 'nm://qwerty:37458/'"
	| destinationName |
	destinationName _ (address isString and: [ address beginsWith: 'nm://']) ifTrue: [address asUrl] ifFalse: [address].
	
	^map
		keyAtValue: destinationName
		ifAbsent: [^ nil]! !

!NmWorldMap class methodsFor: 'accessing point' stamp: 'mu 1/5/2003 23:19'!
rightOf: address 
	"NmWorldMap rightOf: 'nm://smalltalk:37458/' asUrl"
	| point destinationPoint |
	point _ self pointOf: address.
	point isNil
		ifTrue: [^ nil].
	destinationPoint _ Point x: point x + 1 y: point y.
	^ map
		at: destinationPoint
		ifAbsent: [^ nil]! !

!NmWorldMap class methodsFor: 'accessing point' stamp: 'mu 1/5/2003 23:19'!
topOf: address 
	"NmWorldMap topOf: 'nm://smalltalk:37458/' asUrl"
	| point destinationPoint |
	point _ self pointOf: address.
	point isNil
		ifTrue: [^ nil].
	destinationPoint _ Point x: point x y: point y + 1.
	^ map
		at: destinationPoint
		ifAbsent: [^ nil]! !


!NmWorldMap class methodsFor: 'converting' stamp: 'mu 3/17/2003 00:12'!
csvString
	"NmWorldMap csvString"
	^ NmWorldInfoReadWriter csvStringFrom: map! !

!NmWorldMap class methodsFor: 'converting' stamp: 'mu 3/17/2003 01:36'!
fromCSVString: aString 
	| rawDict mapDict |
	rawDict _ NmWorldInfoReadWriter parseCSVString: aString.
	mapDict _ Dictionary new.
	rawDict
		keysAndValuesDo: [:point :worldName | mapDict
				at: (Point readFrom: point)
				put: worldName asUrl].
	self map: mapDict! !


!NmWorldMap class methodsFor: 'prototype' stamp: 'mu 1/17/2003 14:51'!
beTwoNeighbors
	"NmWorldMap beTwoNeighbors"
	| newMap |
	newMap _ Dictionary new.
	newMap at: 0@0 put: NmUrl local.
	newMap at: 1@0 put: (NmUrl local copy port: NmUrl local port + 1).
	self map: newMap.
	^self! !


!NmWorldMap class methodsFor: 'private' stamp: 'mu 3/13/2003 18:27'!
resetLocal
	self map
		keysAndValuesDo: [:point :url | url = NmUrl local
				ifTrue: [url isLocal: true.
					localPoint _ point.
					^ self]].
	"no match"
	localPoint _ 0@0! !


!NmWorldMap class methodsFor: 'accessing name' stamp: 'mu 3/23/2003 15:46'!
allWorldNames
	^map values
	! !

!NmWorldMap class methodsFor: 'accessing name' stamp: 'mu 3/13/2003 19:17'!
mapName
	^NmSettings mapName! !

!NmWorldMap class methodsFor: 'accessing name' stamp: 'mu 3/13/2003 19:18'!
mapName: aString
	^NmSettings mapName: aString! !


!NmWorldMap class methodsFor: 'accessing destination' stamp: 'mu 3/23/2003 15:07'!
allDestinations
	"NmWorldMap allDestinations"
	^NmResolver default ipAndPortArrays
		! !

!NmWorldMap class methodsFor: 'accessing destination' stamp: 'mu 3/23/2003 16:19'!
bottomDestination
	"NmWorldMap bottomDestination"
	^NmResolver default ipAndPortArrayAt:  localPoint + (0@-1)
		! !

!NmWorldMap class methodsFor: 'accessing destination' stamp: 'mu 3/23/2003 16:19'!
leftDestination
	"NmWorldMap leftDestination"
	^NmResolver default ipAndPortArrayAt:  localPoint + (-1@0)
		! !

!NmWorldMap class methodsFor: 'accessing destination' stamp: 'mu 3/23/2003 15:22'!
neighborDestinations
	^(localPoint fourNeighbors collect: [:each | NmResolver default ipAndPortArrayAt: each]) reject: [:each | each isNil]! !

!NmWorldMap class methodsFor: 'accessing destination' stamp: 'mu 3/23/2003 16:19'!
rightDestination
	"NmWorldMap rightDestination"
	^NmResolver default ipAndPortArrayAt:  localPoint + (1@0)
		! !

!NmWorldMap class methodsFor: 'accessing destination' stamp: 'mu 3/23/2003 15:35'!
survivorsDo: aBlock 
	| allDestinations availableDestinations otherDestinations |
	allDestinations _ self allDestinations.
	availableDestinations _ NmAvailabilityChecker default availableDestinations.
	(self neighborDestinations intersection: availableDestinations)
		do: [:each | aBlock value: each].
	otherDestinations _ allDestinations removeAll: self neighborDestinations.
	(otherDestinations intersection: availableDestinations)
		do: [:each | aBlock value: each]! !

!NmWorldMap class methodsFor: 'accessing destination' stamp: 'mu 3/23/2003 16:19'!
topDestination
	"NmWorldMap topDestination"
	^NmResolver default ipAndPortArrayAt:  localPoint + (0@1)
		! !

NmWorldMap initialize!